import typing as t

from moviebox_api.v1._bases import BaseContentProviderAndHelper
from moviebox_api.v1.helpers import assert_instance
from moviebox_api.v2.exceptions import InvalidDetailPathError
from moviebox_api.v2.helpers import get_absolute_url, validate_detail_path
from moviebox_api.v2.models import SpecificItemDetailsModel
from moviebox_api.v2.requests import Session


class BaseItemDetails(BaseContentProviderAndHelper):
    """Base class for specific movie/tv-series (item) details"""

    api_endpoint = get_absolute_url("/wefeed-h5api-bff/detail")

    def __init__(self, session: Session):
        """Constructor for `BaseItemDetails`

        Args:
            detail_path (str): Specific item detail path
            session (Session): MovieboxAPI request session
        """
        assert_instance(session, Session, "session")
        self._session = session

    def _validate_detail_path(self, detail_path: str) -> t.NoReturn:
        if not validate_detail_path(detail_path):
            raise InvalidDetailPathError(
                f"Invalid detail path passed {detail_path!r} "
                "Recheck and try again"
            )

    async def get_content(self, detail_path: str) -> dict:
        self._validate_detail_path(detail_path)
        content = await self._session.get_from_api(
            self.api_endpoint, params={"detailPath": detail_path}
        )
        return content

    async def get_content_model(
        self, detail_path: str, **kwargs
    ) -> SpecificItemDetailsModel:

        content = await self.get_content(detail_path, **kwargs)
        modelled_content = SpecificItemDetailsModel(**content)

        return modelled_content



"""V2 Models"""

from json import loads
from typing import Any

from pydantic import BaseModel, HttpUrl, field_validator

from moviebox_api.v1.extractor.models.json import (
    MetadataModel,
    PostListModel,
    ResourceModel,
    StarsModel,
)
from moviebox_api.v1.models import (
    ContentCategoryModel,
    ContentImageModel,
    ContentModel,
    PlatformsModel,
    SearchResultsItem as SearchResultsItemV1,
    SearchResultsPagerModel,
)
from moviebox_api.v2.helpers import get_absolute_url


class ContentModelV2(ContentModel):
    """`homepage.operatingList[0].banner.items[0]`"""

    subject: "SearchResultsItem"
    detailPath: str
    url: HttpUrl | None = None

    @field_validator("url", mode="before")
    def validate_url(value: str):
        return value if bool(value) else None


class ContentCategoryBannerModelV2(BaseModel):
    """`homepage.operatingList[0].banner`"""

    items: list[ContentModelV2]  # list of series/movies


class ContentCategoryModelV2(ContentCategoryModel):
    """`homepage.operatingList[0]`"""

    banner: ContentCategoryBannerModelV2 | None = None
    filters: list[Any]
    customData: Any
    genreTopId: str
    detailPath: str


class HomepageContentModel(BaseModel):
    """homepage"""

    platformList: list[PlatformsModel]
    operatingList: list[ContentCategoryModelV2]


class OPS(BaseModel):
    """`SearchResultsModel.items[0].ops`"""

    trace_id: str
    search_abt: str
    q: str


class DubModel(BaseModel):
    """`SearchResultsModel.items[0].dubs[0]`"""

    subjectId: str
    lanName: str
    lanCode: str
    original: bool
    type: int
    detailPath: str


class SearchResultsItem(SearchResultsItemV1):
    """`SearchResultsModel.items[0]`"""

    subtitles: list[str] | None
    ops: OPS | None
    imdbRatingCount: int | None = None
    stills: ContentImageModel | None = None
    postTitle: str
    season: int
    dubs: list[DubModel] | None = None

    @field_validator("ops", mode="before")
    def validate_ops(value: str) -> dict:
        if not value:
            return

        return loads(value)

    @field_validator("subtitles", mode="before")
    def validate_subtitles(value: str) -> list[str]:
        if not value:
            return

        return value.split(",")

    @property
    def page_url(self) -> str:
        """Url to the specific item details"""
        return get_absolute_url(
            f"/wefeed-h5api-bff/detail?detailPath={self.detailPath}"
        )


class SearchResultsModel(BaseModel):
    """Whole search results"""

    pager: SearchResultsPagerModel
    items: list[SearchResultsItem]

    @property
    def first_item(self) -> SearchResultsItem:
        return self.items[0]


class SpecificItemDetailsModel(BaseModel):
    """For all subjectTypes"""

    subject: SearchResultsItem
    stars: list[StarsModel]
    resource: ResourceModel
    metadata: MetadataModel
    isForbid: bool
    watchTimeLimit: int
    postList: PostListModel



"""V2 Helpers"""

import re

import moviebox_api.v1.helpers
from moviebox_api.v2.constants import HOST_URL

VALID_DETAIL_PATH_PATTERN = re.compile(r"^[\w-]+-\w{11}$")


def get_absolute_url(relative_url: str, base_url: str = HOST_URL):

    return moviebox_api.v1.helpers.get_absolute_url(relative_url, base_url)


def validate_detail_path(path: str) -> bool:
    return VALID_DETAIL_PATH_PATTERN.match(path) is not None



"""For finding downloadable media files
and initiating actual download
"""

from abc import ABCMeta

from moviebox_api.v1._bases import BaseContentProviderAndHelper
from moviebox_api.v1.download import CaptionFileDownloader, MediaFileDownloader
from moviebox_api.v1.helpers import assert_instance
from moviebox_api.v1.models import DownloadableFilesMetadata
from moviebox_api.v2.constants import SubjectType
from moviebox_api.v2.helpers import get_absolute_url
from moviebox_api.v2.models import SearchResultsItem
from moviebox_api.v2.requests import Session

__all__ = [
    "CaptionFileDownloader",
    "MediaFileDownloader",
    "DownloadableSingleFilesDetail",
    "DownloadableMovieFilesDetail",
    "DownloadableMusicFilesDetail",
    "DownloadableAnimeFilesDetail",
    "DownloadableEducationFilesDetail",
    "DownloadableTVSeriesFilesDetail",
]


class ImmutableMeta(ABCMeta):
    def __setattr__(cls, name, value):
        if name == "_subject_types":
            raise AttributeError("_subject_types is immutable")

        super().__setattr__(name, value)


class BaseDownloadableFilesDetail(
    BaseContentProviderAndHelper, metaclass=ImmutableMeta
):
    """Base class for fetching and modelling downloadable files detail"""

    _url = get_absolute_url(r"/wefeed-h5api-bff/subject/download")

    _subject_types: tuple[SubjectType] = None
    """Enforce item to be of this subjectType(s). Defaults to None"""

    def __init__(
        self,
        session: Session,
        item: SearchResultsItem,
    ):
        """Constructor for `BaseDownloadableFilesDetail`

        Args:
            session (Session): MovieboxAPI request session.
            item (SearchResultsItem): Movie/TVSeries item
                to handle
        """
        assert_instance(session, Session, "session")
        assert_instance(item, SearchResultsItem, "item")

        if self._subject_types is not None:
            if item.subjectType not in self._subject_types:
                raise ValueError(
                    "item needs to be /any/ of the subjectType(s) "
                    f"{self._subject_types!r}",
                    f"not {item.subjectType!r}",
                )

        self.session = session
        self._item = item

    def _create_request_params(self, season: int, episode: int) -> dict:
        """Creates request parameters

        Args:
            season (int): Season number of the series.
            episde (int): Episode number of the series.
        Returns:
            t.Dict: Request params
        """
        return {
            "subjectId": self._item.subjectId,
            "se": season,
            "ep": episode,
            "detailPath": self._item.detailPath,
        }

    async def get_content(self, season: int, episode: int) -> dict:
        """Performs the actual fetching of files detail.

        Args:
            season (int): Season number of the series.
            episde (int): Episode number of the series.

        Returns:
            t.Dict: File details
        """

        content = await self.session.get_from_api(
            url=self._url,
            params=self._create_request_params(season, episode),
        )
        return content

    async def get_content_model(
        self, season: int, episode: int
    ) -> DownloadableFilesMetadata:
        """Get modelled version of the downloadable files detail.

        Args:
            season (int): Season number of the series.
            episde (int): Episode number of the series.

        Returns:
            DownloadableFilesMetadata: Modelled file details
        """
        contents = await self.get_content(season, episode)
        return DownloadableFilesMetadata(**contents)


class DownloadableSingleFilesDetail(BaseDownloadableFilesDetail):
    """Fetches and model movie/music/anime/education files detail"""

    _subject_types = (
        SubjectType.MOVIES,
        SubjectType.ANIME,
        SubjectType.MUSIC,
        SubjectType.EDUCATION,
    )

    async def get_content(self) -> dict:
        """Actual fetch of files detail"""
        return await super().get_content(season=0, episode=0)

    async def get_content_model(self) -> DownloadableFilesMetadata:
        """Modelled version of the files detail"""
        contents = await self.get_content()
        return DownloadableFilesMetadata(**contents)


DownloadableMovieFilesDetail = DownloadableMusicFilesDetail = (
    DownloadableAnimeFilesDetail
) = DownloadableEducationFilesDetail = DownloadableSingleFilesDetail


class DownloadableTVSeriesFilesDetail(BaseDownloadableFilesDetail):
    """Fetches and model tv-series files detail"""

    _subject_types = (SubjectType.TV_SERIES,)

    # NOTE: Already implemented by parent class - BaseDownloadableFilesDetail



"""Contain support functions and constant variables"""

import logging
import random
import subprocess

import click
from httpx import ConnectTimeout, HTTPStatusError
from pydantic import ValidationError
from throttlebuster import DownloadedFile, DownloadMode

from moviebox_api import __repo__
from moviebox_api.v1 import logger
from moviebox_api.v1.constants import (
    DOWNLOAD_REQUEST_HEADERS,
    ENVIRONMENT_HOST_KEY,
    HOST_URL,
    MIRROR_HOSTS,
    SubjectType,
)
from moviebox_api.v1.core import Search, Session
from moviebox_api.v1.exceptions import (
    ZeroCaptionFileError,
    ZeroSearchResultsError,
)
from moviebox_api.v1.models import (
    CaptionFileMetadata,
    DownloadableFilesMetadata,
    SearchResultsItem,
)

command_context_settings = dict(auto_envvar_prefix="MOVIEBOX")


async def perform_search_and_get_item(
    session: Session,
    title: str,
    year: int,
    subject_type: SubjectType,
    yes: bool,
    search: Search = None,
    message: str = "Select",
) -> SearchResultsItem:
    """Search movie/tv-series etc and return target search results item

    Args:
        session (Session): MovieboxAPI requests session.
        title (str): Partial or complete name of the movie/tv-series.
        year (int): `ReleaseDate.year` filter of the search result items.
        subject_type (SubjectType): Movie or tv-series.
        yes (bool): Proceed with the first item instead of prompting confirmation.
        search (Search, optional): Search object. Defaults to None.
        message (str, optional): Prefix message for the prompt.
            Defaults to "select".

    Raises:
        RuntimeError: When all items are exhausted without a match.

    Returns:
        SearchResultsItem: Targeted movie/tv-series
    """
    search = search or Search(session, title, subject_type)
    search_results = await search.get_content_model()
    subject_type_name = " ".join(subject_type.name.lower().split("_"))

    logger.info(
        f"Query '{title}' yielded {
            'over ' if search_results.pager.hasMore else ''
        }"
        f"{len(search_results.items)} {subject_type_name}."
    )
    items = (
        filter(
            lambda item: item.releaseDate.year == year,
            search_results.items,
        )
        if bool(year)
        else search_results.items
    )
    if not isinstance(items, list):
        items = [item for item in items]

    if yes:
        for item in items:
            # Just iterate once
            return item
    else:
        for pos, item in enumerate(items, start=1):
            if click.confirm(
                f"> {message} ({pos}/{len(items)}) : "
                f"{
                    '[' + item.subjectType.name + '] '
                    if subject_type is SubjectType.ALL
                    else ''
                }{item.title}"
                f" {item.releaseDate.year, item.imdbRatingValue}"
            ):
                return item

    if search_results.pager.hasMore:
        next_search: Search = search.next_page(search_results)
        print(f" Loading next page ({next_search._page}) ...", end="\r")

        logging.info(
            f"Navigating to the search results of page number {next_search._page}"
        )
        return await perform_search_and_get_item(
            session=session,
            title=title,
            year=year,
            subject_type=subject_type,
            yes=yes,
            search=next_search,
        )

    raise RuntimeError(
        "All items in the search results are exhausted. Try researching with"
        " a different keyword"
        f"{' or different year filter.' if year > 0 else ''}"
    )


def get_caption_file_or_raise(
    downloadable_details: DownloadableFilesMetadata, language: str
) -> CaptionFileMetadata:
    """Get caption-file based on desired language or raise ValueError if
    it doesn't exist.

    Args:
        downloadable_details (DownloadableFilesMetadata)
        language (str): language filter such as `en` or `English`

    Raises:
        ValueError: Incase caption file for target does not exist.
        NoCaptionFileError: Incase the items lack any caption file.

    Returns:
        CaptionFileMetadata: Target caption file details
    """
    target_caption_file = downloadable_details.get_subtitle_by_language(language)

    if target_caption_file is None:
        language_subtitle_map = (
            downloadable_details.get_language_short_subtitle_map
            if len(language) == 2
            else downloadable_details.get_language_subtitle_map
        )
        subtitle_language_keys = list(language_subtitle_map().keys())

        if subtitle_language_keys:
            raise ValueError(
                f"There is no caption file for the language '{language}'. "
                f"Choose from available ones - {
                    ', '.join(list(subtitle_language_keys))
                }"
            )
        else:
            raise ZeroCaptionFileError(
                "The target item has no any caption file. Use --no-caption or "
                "--ignore-missing-caption flags"
                " if you're using the commandline interface to suppress "
                "this error."
            )
    return target_caption_file


def prepare_start(
    quiet: bool = False, verbose: int = 0, host_url: str = HOST_URL
) -> None:
    """Set up some stuff for better CLI usage such as:

    - Set higher logging level for some packages.
    ...

    """
    if verbose > 3:
        verbose = 2

    logging.basicConfig(
        format=(
            "[%(asctime)s] : %(levelname)s - %(message)s"
            if verbose
            else "[%(module)s] %(message)s"
        ),
        datefmt="%d-%b-%Y %H:%M:%S",
        level=(
            logging.ERROR
            if quiet
            # just a hack to ensure
            #           -v -> INFO
            #           -vv -> DEBUG
            else (30 - (verbose * 10))
            if verbose > 0
            else logging.INFO
        ),
    )
    logging.info(f"Using host url - {host_url}")

    packages = ("httpx",)

    for package_name in packages:
        package_logger = logging.getLogger(package_name)
        package_logger.setLevel(logging.WARNING)


def process_download_runner_params(params: dict) -> dict:
    """Format parsed args from cli to required types and add extra ones

    Args:
        params (dict): Parameters for `Downloader.run`

    Returns:
        dict: Processed parameters
    """
    params["mode"] = DownloadMode.map().get(params.get("mode").lower())

    return params


def show_any_help(exception: Exception, exception_msg: str) -> int:
    """Process exception and suggest solution if exists.

    Args:
        exception (Exception): Exact exception encountered.
        exception_msg (str): Exception message

    Returns:
        int: Exit status code
    """
    exit_code = 1

    if isinstance(exception, ConnectTimeout):
        logging.info(
            "Internet connection request has timed out. Check your connection"
            " and retry."
        )

    elif isinstance(exception, HTTPStatusError):
        match exception.response.status_code:
            case 403:
                logging.info(
                    "Looks like you're in a region that Moviebox doesn't offer"
                    " their services to. "
                    "Use a proxy or a VPN from a different geographical location"
                    " to bypass this restriction."
                )

    elif isinstance(exception, ValidationError):
        logging.info(
            "Looks like there are structural changes in the server response.\n"
            f"Report this issue at {__repo__}/issues/new"
        )

    if "404 Domain" in exception_msg:
        example_host = random.choice(MIRROR_HOSTS)
        logging.info(
            'Run "moviebox-v1 mirror-hosts" command to check available mirror'
            " hosts"
            " and "
            "then export it to the environment using name "
            f'{ENVIRONMENT_HOST_KEY}".\n'
            "For instance: In *nix systems you might run export "
            f'{ENVIRONMENT_HOST_KEY}="{example_host}"'
            f' while in Windows : "set MOVIEBOX_API_HOST={example_host}'
        )

    if not isinstance(
        exception,
        (
            ValueError,
            AssertionError,
            RuntimeError,
            ZeroCaptionFileError,
            ZeroSearchResultsError,
        ),
    ):
        logging.info(
            "Incase the error persist then feel free to submit the issue at"
            f" {__repo__}/issues/new"
        )

    return exit_code


def stream_video_via_mpv(
    url: str, subtitle_details_items: list[DownloadedFile], subtitles_dir: str
):
    try:
        # Create an MPV command with properly formatted headers
        mpv_cmd = ["mpv"]
        # Disable youtube-dl/yt-dlp since we're streaming a direct video URL
        mpv_cmd.append("--no-ytdl")
        for header_name, header_value in DOWNLOAD_REQUEST_HEADERS.items():
            mpv_cmd.append(f"--http-header-fields={header_name}: {header_value}")

        for index, sub_file in enumerate(subtitle_details_items):
            if index == 0:
                mpv_cmd.append("--sid=1")
            # Convert to absolute path for mpv compatibility
            subtitle_path = sub_file.saved_to.resolve().as_posix()
            mpv_cmd.append(f"--sub-file={subtitle_path}")

        mpv_cmd.append(str(url))

        logging.info("Launching MPV with required headers and subtitles...")

        logging.debug(f"MPV launch commands :  {' '.join(mpv_cmd)}")

        subprocess.run(mpv_cmd)

        # shutil.rmtree(subtitles_dir, ignore_errors=True)

        return (None, None)

    except FileNotFoundError as e:
        raise Exception(
            "MPV player not found. Please install it from "
            "https://mpv.io/installation/ "
            'to use streaming feature or retry using "--stream-via vlc" instead.'
        ) from e

    except Exception as e:
        raise Exception(f"Error launching MPV: {e}") from e


def stream_video_via_vlc(
    url: str, subtitle_details_items: list[DownloadedFile], subtitles_dir: str
):
    try:
        user_agent = DOWNLOAD_REQUEST_HEADERS["User-Agent"]
        referrer = DOWNLOAD_REQUEST_HEADERS["Referer"]

        mpv_cmd = [
            "vlc",
            f":http-user-agent={user_agent}",
            f"--http-referrer={referrer}",
        ]

        for sub_file in subtitle_details_items:
            subtitle_path = sub_file.saved_to.resolve().as_posix()
            mpv_cmd.append(f"--sub-file={subtitle_path}")

        mpv_cmd.append(str(url))

        logging.info("Launching VLC with required headers and subtitles...")

        logging.debug(f"VLC launch commands :  {' '.join(mpv_cmd)}")

        subprocess.run(mpv_cmd)

        # shutil.rmtree(subtitles_dir, ignore_errors=True)
        # TODO: Reconsider clearing subtitle folder
        # - Since its saved on temp-folder then OS will handle that...?

        return (None, None)

    except FileNotFoundError as e:
        raise Exception(
            "VLC media player not found. Please install it from "
            "https://www.videolan.org "
            'to use streaming feature or retry using "--stream-via mpv" instead.'
        ) from e

    except Exception as e:
        raise Exception(f"Error launching VLC: {e}") from e


media_player_name_func_map = {
    "mpv": stream_video_via_mpv,
    "vlc": stream_video_via_vlc,
}



"""
Main module for v2 submodule.
Generate models from httpx request responses.
Also provides object mapping support to specific extracted item details
"""

import moviebox_api.v1.core
from moviebox_api.v1.helpers import assert_instance
from moviebox_api.v2._bases import BaseItemDetails
from moviebox_api.v2.constants import SINGLE_ITEM_SUBJECT_TYPES, SubjectType
from moviebox_api.v2.helpers import get_absolute_url
from moviebox_api.v2.models import (
    HomepageContentModel,
    SearchResultsItem,
    SearchResultsModel,
    SpecificItemDetailsModel,
)
from moviebox_api.v2.requests import Session


class Homepage(moviebox_api.v1.core.Homepage):
    _url = get_absolute_url("/wefeed-h5api-bff/home?host=moviebox.ph")

    async def get_content_model(self) -> HomepageContentModel:
        """Modelled version of the contents"""
        content = await self.get_content()
        return HomepageContentModel(**content)


class SearchSuggestion(moviebox_api.v1.core.SearchSuggestion):
    _url = get_absolute_url("/wefeed-h5api-bff/subject/search-suggest")


class Search(moviebox_api.v1.core.Search):
    _url = get_absolute_url("/wefeed-h5api-bff/subject/search")

    async def get_content_model(self) -> SearchResultsModel:
        """Modelled version of the contents.

        Returns:
            SearchResultsModel: Modelled contents
        """
        contents = await self.get_content()
        return SearchResultsModel(**contents)


class ItemDetails(BaseItemDetails):
    """Fetch specific item details - movies, anime, education,
    music & tv-series"""

    def __init__(self, session: Session):
        """Constructor for `SingleItemDetails`

        Args:
            session (Session): MovieboxAPI request session
        """
        super().__init__(session)

    async def get_content(self, path_or_item: str | SearchResultsItem) -> dict:
        """Get specific item details

        Args:
            path_or_item (str|SearchResultsItem): Detail path for specific item
              page or search-results-item.

        Raises:
            ValueError: InvalidDetailPathError
        """

        assert_instance(path_or_item, (str, SearchResultsItem), "path_or_item")

        detail_path = path_or_item

        if isinstance(path_or_item, SearchResultsItem):
            detail_path = path_or_item.detailPath

        return await super().get_content(detail_path)

    async def get_content_model(
        self, path_or_item: str | SearchResultsItem, **kwargs
    ) -> SpecificItemDetailsModel:

        content = await self.get_content(path_or_item, **kwargs)
        return SpecificItemDetailsModel(**content)


class SingleItemDetails(BaseItemDetails):
    """Fetch specific item details - movies, anime, education, music"""

    def __init__(self, session: Session):
        """Constructor for `SingleItemDetails`

        Args:
            session (Session): MovieboxAPI request session
        """
        super().__init__(session)

    async def get_content(self, path_or_item: str | SearchResultsItem) -> dict:
        """Get specific item details

        Args:
            path_or_item (str|SearchResultsItem): Detail path for specific item
              page or search-results-item.

        Raises:
            ValueError: InvalidDetailPathError
        """

        assert_instance(path_or_item, (str, SearchResultsItem), "path_or_item")

        detail_path = path_or_item

        if isinstance(path_or_item, SearchResultsItem):
            if path_or_item.subjectType == SubjectType.TV_SERIES:
                raise ValueError(
                    "item needs to be any of the following subjectTypes"
                    f"{SINGLE_ITEM_SUBJECT_TYPES!r} "
                    f"not {path_or_item.subjectType!r}"
                )

            detail_path = path_or_item.detailPath

        return await super().get_content(detail_path)

    async def get_content_model(
        self, path_or_item: str | SearchResultsItem, **kwargs
    ) -> SpecificItemDetailsModel:

        content = await self.get_content(path_or_item, **kwargs)
        return SpecificItemDetailsModel(**content)


MusicDetails = AnimeDetails = EducationDetails = MovieDetails = SingleItemDetails


class TVSeriesDetails(BaseItemDetails):
    """Fetch specific item details - tv_series"""

    def __init__(self, session: Session):
        """Constructor for `TVSeriesItemDetails`

        Args:
            session (Session): MovieboxAPI request session
        """
        super().__init__(session)

    async def get_content(self, path_or_item: str | SearchResultsItem) -> dict:
        """Get specific item details

        Args:
            path_or_item (str|SearchResultsItem): Detail path for specific item
              page or search-results-item.

        Raises:
            ValueError: InvalidDetailPathError
        """

        assert_instance(path_or_item, (str, SearchResultsItem), "path_or_item")

        detail_path = path_or_item

        if isinstance(path_or_item, SearchResultsItem):
            if path_or_item.subjectType != SubjectType.TV_SERIES:
                raise ValueError(
                    f"item needs to be of subjectType"
                    f"{SubjectType.TV_SERIES!r} only"
                    f"not {path_or_item.subjectType!r}"
                )

            detail_path = path_or_item.detailPath

        return await super().get_content(detail_path)

    async def get_content_model(
        self, path_or_item: str | SearchResultsItem, **kwargs
    ) -> SpecificItemDetailsModel:

        content = await self.get_content(path_or_item, **kwargs)
        return SpecificItemDetailsModel(**content)



"""For server interaction"""

from httpx._config import DEFAULT_TIMEOUT_CONFIG
from httpx._types import (
    CookieTypes,
    ProxyTypes,
    TimeoutTypes,
)

import moviebox_api.v1.requests
from moviebox_api.v2.constants import DOWNLOAD_REQUEST_HEADERS

request_cookies = {}

__all__ = ["Session"]


class Session(moviebox_api.v1.requests.Session):
    _moviebox_app_info_url = None

    def __init__(
        self,
        headers: ProxyTypes | None = DOWNLOAD_REQUEST_HEADERS,
        cookies: CookieTypes | None = request_cookies,
        timeout: TimeoutTypes = DEFAULT_TIMEOUT_CONFIG,
        proxy: ProxyTypes | None = None,
        **httpx_kwargs,
    ):
        """Constructor for `Session`

        Args:
            headers (ProxyTypes | None, optional): Http request headers. Defaults to DOWNLOAD_REQUEST_HEADERS.
            cookies (CookieTypes | None , optional): Http request cookies. Defaults to request_cookies.
            timeout (TimeoutTypes, optional): Http request timeout in seconds. Defaults to DEFAULT_TIMEOUT_CONFIG.
            proxy (ProxyTypes | None, optional): Http requests proxy. Defaults to None.

        httpx_kwargs : Other keyword arguments for `httpx.AsyncClient`
        """  # noqa: E501

        super().__init__(
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            proxy=proxy,
            **httpx_kwargs,
        )

    async def ensure_cookies_are_assigned(self) -> bool:
        return True

    async def _fetch_app_info(self) -> None:
        raise NotImplementedError(
            "This functionality is only available to V1 of moviebox_api.requests"
        )



import logging
import os
import sys

import click

from moviebox_api.utils import build_command_group
from moviebox_api.v1.cli.helpers import show_any_help
from moviebox_api.v1.cli.interface import get_commands_map
from moviebox_api.v2.cli.interface import get_commands_map as get_commmands_map_2


@click.group()
@click.version_option(package_name="moviebox-api")
def cli_entry():
    """Search and download movies/tv-series and their subtitles.
    envvar-prefix : MOVIEBOX"""


@cli_entry.group()
def v1():
    """Search and download movies/tv-series using moviebox-API v1"""


@cli_entry.group()
def v2():
    """Search and download movies/tv-series etc using moviebox-API v2"""


build_command_group(v1, get_commands_map())
build_command_group(v2, get_commmands_map_2())


def main():
    try:
        return cli_entry()

    except Exception as e:
        exception_msg = str({e.args[1] if e.args and len(e.args) > 1 else e})

        DEBUG = os.getenv("DEBUG", "0") == "1"

        if DEBUG:
            logging.exception(e)
        else:
            if bool(exception_msg):
                logging.error(exception_msg)
            sys.exit(show_any_help(e, exception_msg))

    sys.exit(1)



import pytest

from moviebox_api.v1.core import (
    MovieDetails,
    Search,
    SubjectType,
    TVSeriesDetails,
)
from moviebox_api.v1.download import (
    CaptionFileDownloader,
    DownloadableMovieFilesDetail,
    DownloadableTVSeriesFilesDetail,
    MediaFileDownloader,
)
from moviebox_api.v1.requests import Session


@pytest.mark.asyncio
async def test_download_movie_caption_file():
    session = Session()
    search = Search(session, "avatar", subject_type=SubjectType.MOVIES)

    search_results = await search.get_content_model()
    target_movie = search_results.first_item

    # We just fetch page details of that specific item.
    # It would make much sense when you cache the item-page so you wont be
    # required to start afresh come next time, rather proceed where you
    # stopped from.

    # To make this test future proof to changes on the backend side,
    # we just have to fetch them from the server instead of using the offline one.

    target_movie_details_instance = MovieDetails(target_movie, session)
    # Alternatively :
    # target_movie_details_instance = search.get_item_details(target_movie)

    target_movie_details_model = (
        await target_movie_details_instance.get_content_model()
    )

    downloadable_files = DownloadableMovieFilesDetail(
        session, target_movie_details_model
    )
    downloadable_files_detail = await downloadable_files.get_content_model()
    target_caption_file = downloadable_files_detail.english_subtitle_file

    caption_file_downloader = CaptionFileDownloader()
    response = await caption_file_downloader.run(
        target_caption_file,
        filename=target_movie.title + "- English.srt",
        test=True,
    )
    assert response.is_success


@pytest.mark.asyncio
async def test_download_movie_file():
    session = Session()
    search = Search(session, "avatar", subject_type=SubjectType.MOVIES)
    search_results = await search.get_content_model()
    target_movie = search_results.first_item

    #  We just fetch page details of that specific item
    target_movie_details_instance = MovieDetails(target_movie, session)
    target_movie_details_model = (
        await target_movie_details_instance.get_content_model()
    )

    downloadable_files = DownloadableMovieFilesDetail(
        session, target_movie_details_model
    )
    downloadable_files_detail = await downloadable_files.get_content_model()
    target_media_file = downloadable_files_detail.best_media_file

    media_file_downloader = MediaFileDownloader()
    response = await media_file_downloader.run(
        target_media_file, filename=target_movie.title + ".mp4", test=True
    )
    assert response.is_success


@pytest.mark.asyncio
async def test_download_tv_series_caption_file():
    session = Session()
    search = Search(session, "Merlin", subject_type=SubjectType.TV_SERIES)
    search_results = await search.get_content_model()
    target_series = search_results.first_item

    #  We just fetch page details of that specific item
    target_series_details_instance = TVSeriesDetails(target_series, session)
    target_series_details_model = (
        await target_series_details_instance.get_content_model()
    )

    downloadable_files = DownloadableTVSeriesFilesDetail(
        session, target_series_details_model
    )

    downloadable_files_detail = await downloadable_files.get_content_model(
        season=1, episode=1
    )
    target_caption_file = downloadable_files_detail.english_subtitle_file

    caption_file_downloader = CaptionFileDownloader()
    response = await caption_file_downloader.run(
        target_caption_file, filename=target_series, test=True
    )
    assert response.is_success


@pytest.mark.asyncio
async def test_download_tv_series_file():
    session = Session()
    search = Search(session, "Merlin", subject_type=SubjectType.TV_SERIES)
    search_results = await search.get_content_model()
    target_series = search_results.first_item

    #  We just fetch page details of that specific item
    target_series_details_instance = TVSeriesDetails(target_series, session)
    target_series_details_model = (
        await target_series_details_instance.get_content_model()
    )

    downloadable_files = DownloadableTVSeriesFilesDetail(
        session, target_series_details_model
    )
    downloadable_files_detail = await downloadable_files.get_content_model(
        season=1, episode=1
    )
    target_media_file = downloadable_files_detail.best_media_file

    media_file_downloader = MediaFileDownloader()
    response = await media_file_downloader.run(
        target_media_file, filename=target_series, test=True
    )
    assert response.is_success



"""This module stores constant variables"""

import os

from moviebox_api.v1.constants import SubjectType
from moviebox_api.v2 import logger

"""asyncio event loop"""

MIRROR_HOSTS = ("h5-api.aoneroom.com",)
"""Mirror domains/subdomains of Moviebox-api"""

ENVIRONMENT_HOST_KEY = "MOVIEBOX_API_HOST_V2"
"""User declares host to use as environment variable using this key"""

SELECTED_HOST = (
    os.getenv(ENVIRONMENT_HOST_KEY) or MIRROR_HOSTS[0]
)  # TODO: Choose the right value based on working status
"""Host adress only with protocol"""

HOST_PROTOCOL = "https"
"""Host protocol i.e http/https"""

HOST_URL = f"{HOST_PROTOCOL}://{SELECTED_HOST}/"
"""Complete host adress with protocol"""

logger.info(f"Moviebox API host url - {HOST_URL}")

REFERER = "https://videodownloader.site/"

DEFAULT_REQUEST_HEADERS = {
    "X-Client-Info": '{"timezone":"Africa/Nairobi"}',
    # TODO: Set this value dynamically.
    "Accept-Language": "en-US,en;q=0.5",
    "Accept": "application/json",
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:137.0) Gecko/20100101"
    " Firefox/137.0",
    "Referer": REFERER,
}
"""For general http requests other than download"""

DOWNLOAD_REQUEST_REFERER = REFERER

DOWNLOAD_REQUEST_HEADERS = {
    "Accept": "*/*",  # "video/webm,video/ogg,video/*;q=0.9,application/ogg;q=0.7,
    # audio/*;q=0.6,*/*;q=0.5",
    "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:137.0) Gecko/20100101 "
    "Firefox/137.0",
    "Origin": REFERER,
    "Referer": DOWNLOAD_REQUEST_REFERER,
}
"""For media and subtitle files download requests"""

SINGLE_ITEM_SUBJECT_TYPES = {
    SubjectType.MUSIC,
    SubjectType.MOVIES,
    SubjectType.ANIME,
    SubjectType.EDUCATION,
}



"""
V2 of Moviebox-API
"""

import logging

logger = logging.getLogger(__name__)


logging.getLogger("moviebox_api.v1").setLevel(logging.DEBUG)

from moviebox_api.v2.core import (  # noqa: E402
    AnimeDetails,
    EducationDetails,
    Homepage,
    ItemDetails,
    MovieDetails,
    MusicDetails,
    Search,
    SearchSuggestion,
    SingleItemDetails,
    TVSeriesDetails,
)
from moviebox_api.v2.download import (  # noqa: E402
    DownloadableSingleFilesDetail,
    DownloadableTVSeriesFilesDetail,
)
from moviebox_api.v2.requests import Session  # noqa: E402

__all__ = [
    "Session",
    "DownloadableSingleFilesDetail",
    "DownloadableTVSeriesFilesDetail",
    "DownloadableSingleFilesDetail",
    "DownloadableTVSeriesFilesDetail",
    "Homepage",
    "ItemDetails",
    "Search",
    "SearchSuggestion",
    "SingleItemDetails",
    "TVSeriesDetails",
    "MovieDetails",
    "MusicDetails",
    "AnimeDetails",
    "EducationDetails",
]
